@@ -41,7 +41,7 @@ GEN rhorealform5(GEN x, GEN D, GEN isqrtD, GEN sqrtD);
 GEN redrealform5(GEN x, GEN D, GEN isqrtD, GEN sqrtD, long sens);
 GEN initializeform5(long *ex, GEN **tabform, long cardtab, GEN d, GEN isqrtd, GEN sqrtd, long sens);
 GEN redrealform(GEN x, GEN D, GEN isqrtD, GEN sqrtD, long sens, long precreg);
-GEN gcdrealnoer(GEN a, GEN b);
+GEN gcdrealnoer(GEN a, GEN b, long *pte);
 GEN lfunc(GEN D);
 
 void
@@ -86,7 +86,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
   dr=cgetr(3);affir(D,dr);drc=rtodbl(dr);logd=log(fabs(drc));
   lim=sqrt(fabs(drc)/3.0);cp=(long)exp(sqrt(logd*log(logd)/8.0));
   increaseimag:
-  if(DEBUGLEVEL) fprintferr("cbach = %f\n",cbach);
+  if(DEBUGLEVEL) {fprintferr("cbach = %f\n",cbach);flusherr();}
   nreldep=nrelsup=0;
   limc=(long)(cbach*logd*logd);
   if(cp>limc) limc=cp;
@@ -146,7 +146,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     form=compimag(form,pc);
     fpc=factorisequad(form,kc,limc,&lo,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
     nbtest++;
-    if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+    if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
     if(fpc>1)
     {
       fpd=largeprime(fpc,ex,1+(s%kc),0,mglob,hashtab);
@@ -154,7 +154,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
       {
 	auxrel++;
 	s++;
-	if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	lo1=lo;
 	for(j=1;j<=lo1;j++) {primfact1[j]=primfact[j];expoprimfact1[j]=expoprimfact[j];}
 	form1=vp[1][fpd[2]];
@@ -218,7 +218,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     if(fpc==1)
     {
       s++;
-      if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+      if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
       for(i=1;i<=m;i++) mat[s][numbase[subbase[i]]]= -ex[i];
       mat[s][1+(s-1)%kc]--;
       for(j=1;j<=lo;j++)
@@ -233,7 +233,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>=2) fprintferr("\n");
+    if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
     fprintferr("temps relations mm: ");fprintferr("%ld\n",timer2());flusherr();
     fprintferr("Apres relations mm, s = nbtest = %ld\n",s);flusherr();
   }
@@ -247,7 +247,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     form=compimag(form,pc);
     fpc=factorisequad(form,kc,limc,&lo,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
     nbtest++;
-    if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+    if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
     if(fpc>1)
     {
       fpd=largeprime(fpc,ex,s+1-CO,0,mglob,hashtab);
@@ -255,7 +255,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
       {
 	auxrel++;
 	s++;lo1=lo;
-	if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	for(j=1;j<=lo1;j++) {primfact1[j]=primfact[j];expoprimfact1[j]=expoprimfact[j];}
 	form1=vp[1][fpd[2]];
 	for(i=2;i<=m;i++) 
@@ -318,7 +318,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     if(fpc==1)
     {
       s++;
-      if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+      if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
       for(i=1;i<=m;i++) mat[s][numbase[subbase[i]]]= -ex[i];
       mat[s][s-CO]= -1;
       for(j=1;j<=lo;j++)
@@ -334,7 +334,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>1) fprintferr("\n");      
+    if(DEBUGLEVEL>1) {fprintferr("\n");flusherr();}
     fprintferr("temps pour trouver les relations aleatoires: ");
     fprintferr("%ld\n",timer2());flusherr();
     fprintferr("nbrelations/nbtest = %ld/%ld\n",s,nbtest);flusherr();
@@ -343,7 +343,10 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
   if(DEBUGLEVEL>2)
   {
     if(kc2>kc)
+    {
       fprintferr("be honest for primes from %ld to %ld\n",base[kc+1],base[kc2]);
+    flusherr();
+    }
   }
   while(s<kc2)
   {
@@ -352,7 +355,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     form=vp[1][ex[1]];
     for (i=2;i<=m;i++) form=compimag(form,vp[i][ex[i]]);
     pp=base[s+1];
-    if(DEBUGLEVEL>=2) fprintferr(" %ld",pp);
+    if(DEBUGLEVEL>=2) {fprintferr(" %ld",pp);flusherr();}
     pc=primeform(D,stoi(pp),0);
     form=compimag(form,pc);
     fpc=factorisequad(form,s,pp-1,&lo,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
@@ -376,14 +379,14 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
       else
       {
 	avma=av;cbach=min(2*cbach,6);cbach2=max(cbach2,cbach);
-	if(DEBUGLEVEL>=2) fprintferr("\n");
+	if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
 	goto increaseimag;
       }
     }
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>=2) fprintferr("\n");
+    if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
     fprintferr("temps be honest: ");fprintferr("%ld\n",timer2());flusherr();
   }
   matc=cgetg(kcco+1,19);for(i=1;i<=kcco;i++) matc[i]=lgetg(1,18);
@@ -400,7 +403,10 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     for(i=1;i<=kc;i++) vinvperm[vperm[i]]=i;
     s=0;extrarel=nlze+2;
     if(DEBUGLEVEL>1)
+    {
       fprintferr("recherche de %ld relations supplementaires\n",extrarel);
+      flusherr();
+    }
     extramat=cgetg(extrarel+1,19);
     for(j=1;j<=extrarel;j++) extramat[j]=lgetg(kc+1,18);
     free(ex);ex=(long*)malloc(sizeof(long)*(nlze+1));
@@ -411,11 +417,11 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
       for (i=2;i<=nlze;i++)
 	form=compimag(form,gpuigs(primeform(D,stoi(labs(vectprime[vperm[i]])),0),ex[i]));
       fpc=factorisequad(form,kc,limc,&lo,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
-      if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+      if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
       if(fpc==1)
       {
 	s++;
-	if(DEBUGLEVEL>1) fprintferr(" %ld",s);
+	if(DEBUGLEVEL>1) {fprintferr(" %ld",s);flusherr();}
 	p1=(GEN)extramat[s];
 	for(i=1;i<=nlze;i++) p1[i]=lstoi(-ex[i]);
 	for(i=nlze+1;i<=kc;i++) p1[i]=zero;
@@ -434,7 +440,7 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     for(i=1;i<=extrarel;i++) extramatc[i]=lgetg(1,18);
     if(DEBUGLEVEL)
     {
-      if(DEBUGLEVEL>=2) fprintferr("\n");
+      if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
       fprintferr("temps calcul relations supplementaires: ");fprintferr("%ld\n",timer2());flusherr();
     }
     if(nrelsup) nlze=0;
@@ -475,7 +481,10 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
       for(i=1;i<=m;i++) free(vp[i]);free(vp);
       for(i=1;i<HASHT;i++) freehash(hashtab[i]);
       if(cbach>5.99)
+      {
 	fprintferr("\n  ***   Warning: check is greater than 1.5, suggest increasing extra relations\n");
+	flusherr();
+      }
       else 
       {
 	avma=av;cbach=min(2*cbach,6);cbach2=max(cbach2,cbach);
@@ -484,13 +493,14 @@ buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
     }
     else
     {
-      if(DEBUGLEVEL) fprintferr("\n ***** check = %f\n\n",gtodouble(c_1));
+      if(DEBUGLEVEL)
+      {fprintferr("\n ***** check = %f\n\n",gtodouble(c_1));flusherr();}
       nlze=min(kc,nrelsup);goto morerelimag;
     }
   }
   u1u2=smith2(mit);u1=(GEN)u1u2[1];u2=(GEN)u1u2[2];
   met=gmul(u1,gmul(mit,u2));
-  u1=ginv(u1);
+  u1=reducemodmatrix(ginv(u1),mit);
   lo=lg(met)-1;
   c=0;for(i=1;i<=lo;i++) if(!gcmp1(gcoeff(met,i,i))) c++;
   if(DEBUGLEVEL) {fprintferr("temps smith/groupe de classes: ");fprintferr("%ld\n",timer2());flusherr();}
@@ -657,7 +667,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
 {
   long sens,RELSUP,nrelsup,nreldep,kccopro;
   double cbach,cbach2;
-  long precreg,limc,limbach,cardsubbase,cardsub,nbram,auxrel;
+  long precreg,limc,limbach,cardsubbase,cardsub,nbram,auxrel,e,maxe;
   long nbprimfact,nbprimfact1,fl,fl2,lo,initform5,findecycle,memoirefindecycle;
   long nbrhocourant,nbrhocumule,nrho,*fpd,b1,b2,fpc,pp,ep,b,extrarel,c,limhash;
   long sizebach,*numbase,*base,*subbase;
@@ -687,7 +697,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
   RELSUP=itos(gRELSUP);
   cbach=gtodouble(gcbach);cbach2=gtodouble(gcbach2);
   increasereal:
-  if(DEBUGLEVEL) fprintferr("cbach = %f\n",cbach);
+  if(DEBUGLEVEL) {fprintferr("cbach = %f\n",cbach);flusherr();}
   nreldep=nrelsup=0;
   initbuchreal(D,cbach,cbach2,&precreg,&log2precis,&qqq,&dr,&drc,&logd,&sqrtD,&isqrtD,&lim,&limc,&limbach,prec);
   sizebach=factorbasequad(D,limbach,limc,&numbase,&base,&sizemat,badprim,&nbbadprim);
@@ -745,7 +755,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     {
       av2=avma;
       fpc=factorisequad(form,sizemat,limc,&nbprimfact,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
-      if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+      if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
       if(fpc==1)
       {
 	if(!initform5)
@@ -755,7 +765,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
 	}
 	for(i=1;i<=nbrhocourant;i++) form1=rhorealform5(form1,D,isqrtD,sqrtD);
 	nbrhocourant=0;s++;
-	if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	addcolumnmat(mat,s,cardsub,numbase,subbase,ex,nbprimfact,primfact,expoprimfact,form1,log2precis,vecexpo);
       }
       else if(fpc>1)
@@ -793,7 +803,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
 	    if((b1+b2)==(fpc<<1))
 	    {
 	      s++;
-	      if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	      if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	      addcolumnmat2(mat,s,cardsub,numbase,subbase,ex,nbprimfact1,primfact1,expoprimfact1,form1,fpd,nbprimfact,primfact,expoprimfact,form2,log2precis,vecexpo);
 	    }
 	  }
@@ -831,7 +841,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>=2) fprintferr("\n");
+    if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
     fprintferr("temps relations mm: ");fprintferr("%ld\n",timer2());flusherr();
     fprintferr("Apres relations mm, s = nbtest = %ld\n",s);flusherr();
   }
@@ -846,7 +856,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     {
       fpc=factorisequad(form,sizemat,limc,&nbprimfact,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
       nbtest++;
-      if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+      if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
       if(fpc>1)
       {
 	fpd=largeprime(fpc,ex,s+1-RELSUP,nrho,cardsubbase,hashtab);
@@ -858,7 +868,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
 	  form=comprealform5(form,pc,D,isqrtD,sqrtD,sens);
 	  for(i=1;i<=nrho;i++) form=rhorealform5(form,D,isqrtD,sqrtD);
 	  auxrel++;s++;nbprimfact1=nbprimfact;
-	  if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	  if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	  for(j=1;j<=nbprimfact1;j++)
 	  {primfact1[j]=primfact[j];expoprimfact1[j]=expoprimfact[j];}
 	  form1=tabpowprform[1][fpd[2]];
@@ -940,7 +950,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
       form=comprealform5(form,pc,D,isqrtD,sqrtD,sens);
       for(i=1;i<=nrho;i++) form=rhorealform5(form,D,isqrtD,sqrtD);
       s++;
-      if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+      if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
       for(i=1;i<=cardsub;i++) mat[s][numbase[subbase[i]]]= -ex[i];
       mat[s][s-RELSUP]= -1;
       for(j=1;j<=nbprimfact;j++)
@@ -955,7 +965,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>1) fprintferr("\n");      
+    if(DEBUGLEVEL>1) {fprintferr("\n");flusherr();}
     fprintferr("temps pour trouver les relations aleatoires: ");
     fprintferr("%ld\n",timer2());flusherr();
     fprintferr("nbrelations/nbtest = %ld/%ld\n",s,nbtest);flusherr();
@@ -964,7 +974,10 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
   if(DEBUGLEVEL>2)
   {
     if(sizebach>sizemat)
+    {
       fprintferr("be honest for primes from %ld to %ld\n",base[sizemat+1],base[sizebach]);
+      flusherr();
+    }
   }
   while(s<sizebach)
   {
@@ -972,7 +985,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     av1=avma;
     form=initializeform3(ex,tabpowprform,cardsub,D,isqrtD,sens);
     pp=base[s+1];
-    if(DEBUGLEVEL>=2) fprintferr(" %ld",pp);
+    if(DEBUGLEVEL>=2) {fprintferr(" %ld",pp);flusherr();}
     pc=redrealform3(primeform(D,stoi(pp),precreg),D,isqrtD,sens);
     form=comprealform3(form,pc,D,isqrtD,sens);forminit=form;fl2=0;
     do
@@ -1009,14 +1022,14 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
       if(cbach>5.99)
 	err(talker,"sorry, buchxxx is not able to compute this field PLEASE REPORT!!!");
       avma=av;cbach=min(2*cbach,6);cbach2=max(cbach2,cbach);
-      if(DEBUGLEVEL>=2) fprintferr("\n");
+      if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
       goto increasereal;
     }
     avma=av1;
   }
   if(DEBUGLEVEL)
   {
-    if(DEBUGLEVEL>=2) fprintferr("\n");
+    if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
     fprintferr("temps be honest: ");fprintferr("%ld\n",timer2());flusherr();
   }
   matc=cgetg(sizematcol+1,19);
@@ -1035,7 +1048,10 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     for(i=1;i<=sizemat;i++) vinvperm[vperm[i]]=i;
     s=0;extrarel=nlze+2;
     if(DEBUGLEVEL>1)
+    {
       fprintferr("recherche de %ld relations supplementaires\n",extrarel);
+      flusherr();
+    }
     extramat=cgetg(extrarel+1,19);
     for(j=1;j<=extrarel;j++) extramat[j]=lgetg(sizemat+1,18);
     extramatc=cgetg(extrarel+1,19);
@@ -1049,11 +1065,11 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
       for (i=2;i<=nlzecard;i++)
 	form=compreal(form,gpuigs(primeform(D,stoi(labs(vectprime[vperm[i]])),precreg),ex[i]));
       fpc=factorisequad(form,sizemat,limc,&lo,primfact,expoprimfact,badprim,nbbadprim,base,limhash);
-      if((DEBUGLEVEL>=2)&&(!fpc)) fprintferr(".");
+      if((DEBUGLEVEL>=2)&&(!fpc)) {fprintferr(".");flusherr();}
       if(fpc==1)
       {
 	s++;p1=(GEN)extramat[s];
-	if(DEBUGLEVEL>=2) fprintferr(" %ld",s);
+	if(DEBUGLEVEL>=2) {fprintferr(" %ld",s);flusherr();}
 	for(i=1;i<=nlzecard;i++) p1[i]=lstoi(-ex[i]);
 	for(i=nlzecard+1;i<=sizemat;i++) p1[i]=zero;
 	for(j=1;j<=lo;j++)
@@ -1070,7 +1086,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     }
     if(DEBUGLEVEL)
     {
-      if(DEBUGLEVEL>=2) fprintferr("\n");
+      if(DEBUGLEVEL>=2) {fprintferr("\n");flusherr();}
       fprintferr("temps calcul relations supplementaires: ");fprintferr("%ld\n",timer2());flusherr();
     }
     if(nrelsup) nlze=0;
@@ -1097,8 +1113,19 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
   p1=gun;sizeofmit=lg(mit)-1;
   for(i=1;i<=sizeofmit;i++) p1=mulii(p1,gcoeff(mit,i,i));
   extrarel=col-sizeofmit;
-  reg=gabs(gcoeff(matc,1,1),0);
-  for(i=2;i<=extrarel;i++) reg=gcdrealnoer(gcoeff(matc,1,i),reg);
+  reg=gabs(gcoeff(matc,1,1),0);maxe=0;
+  for(i=2;(i<=extrarel)&&(maxe<=0);i++)
+  {reg=gcdrealnoer(gcoeff(matc,1,i),reg,&e);maxe=(maxe)?max(maxe,e):e;}
+  if(maxe>0)
+  {
+/* c'est idiot de recalculer. A changer */
+    prec=(precreg<<1)-3;
+    free(vperm);free(vectprime);free(ex);free(base);free(numbase);free(subbase);
+    for(i=1;i<=sizematcol;i++) free(matinit[i]);free(matinit);free(mat);
+    for(i=1;i<=cardsub;i++) free(tabpowprform[i]);free(tabpowprform);
+    for(i=1;i<HASHT;i++) freehash(hashtab[i]);
+    avma=av;goto increasereal;
+  }
   if(DEBUGLEVEL) {fprintferr("temps regulateur: ");fprintferr("%ld\n",timer2());flusherr();}
   if(gexpo(reg)<=-3)
   {
@@ -1120,7 +1147,7 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     else
     {
       nlze=min(sizemat,nrelsup);
-      if(DEBUGLEVEL) fprintferr("regulateur nul : \n");
+      if(DEBUGLEVEL) {fprintferr("regulateur nul : \n");flusherr();}
       goto morerelreal;
     }
   }
@@ -1136,7 +1163,10 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
       for(i=1;i<=cardsub;i++) free(tabpowprform[i]);free(tabpowprform);
       for(i=1;i<HASHT;i++) freehash(hashtab[i]);
       if(cbach>5.99)
+      {
 	fprintferr("\n  ***   Warning: check is greater than 1.5, suggest increasing extra relations\n");
+	flusherr();
+      }
       else
       {
 	avma=av;cbach=min(2*cbach,6);cbach2=max(cbach2,cbach);
@@ -1146,13 +1176,14 @@ buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
     else
     {
       nlze=min(sizemat,nrelsup);
-      if(DEBUGLEVEL) fprintferr("\n ***** check = %f\n\n",gtodouble(c_1));
+      if(DEBUGLEVEL)
+      {fprintferr("\n ***** check = %f\n\n",gtodouble(c_1));flusherr();}
       goto morerelreal;
     }
   }
   u1u2=smith2(mit);u1=(GEN)u1u2[1];u2=(GEN)u1u2[2];
   met=gmul(u1,gmul(mit,u2));
-  u1=ginv(u1);
+  u1=reducemodmatrix(ginv(u1),mit);
   lo=lg(met)-1;
   c=0;for(i=1;i<=lo;i++) if(!gcmp1(gcoeff(met,i,i))) c++;
   if(DEBUGLEVEL) {fprintferr("temps smith/groupe de classes: ");fprintferr("%ld\n",timer2());flusherr();}
@@ -1574,7 +1605,7 @@ initbuchreal(GEN D, double cbach, double cbach2, long *precreg, GEN *log2precis,
 {
   long cp;
 	 
-      /* precision en digits decimaux=(#digits decimaux de D)+50 */
+      /* precision en digits decimaux=2*(#digits decimaux de D)+50 */
       /* ici CBUCH=15,q=27 */
       /* on prendra les p decomposes tels que prod(p)>lim dans la subbase */
       /* limc=Max(c.(log(D))^2,exp((1/8).sqrt(log(D).loglog(D)))) */
@@ -1585,7 +1616,7 @@ initbuchreal(GEN D, double cbach, double cbach2, long *precreg, GEN *log2precis,
       /* nbram est le nombre de p divisant D elimines dans subbase */
       /* tabpowprform est la table des puissances des formes dans tabprform */
 
-  *precreg=max(prec+1,(gexpo(D)>>TWOPOTBITS_IN_LONG)+5);
+  *precreg=max(prec+1,2*(gexpo(D)>>TWOPOTBITS_IN_LONG)+5);
   *log2precis=glog(gdeux,*precreg);
   (*qqq)=BITS_IN_RANDOM-1-(long)ceil(log((double)CBUCH)/log(2.0));
   *dr=cgetr(3);affir(D,*dr);
