@@ -188,7 +188,7 @@ coinit(long grandmot)
   *p = 0;
   do {*--p = grandmot%10 + '0'; grandmot /= 10;} while (grandmot);
   pariputs(p);
-  return cha - p + 9;
+  return (cha - p) + 9;
 }
 
 static void
@@ -278,22 +278,21 @@ ecrireall(GEN x, char format, long dec, long chmp, long flbl)
         else
 	      /* reel non nul */
 	{
-	  if(expo(x)>=((lg(x)-2)<<TWOPOTBITS_IN_LONG))
+	  avmacourant=avma;
+	  if (sgn!=1) pariputc('-');
+	      /*  on arrondit si il y a lieu */
+	  for (i=0;i<=2;i++) arrondi[i]=x[i];
+	  setlg(arrondi,3);
+	  if (dec>0)
+	  {
+	    arrondi[1]=(long)(arrondi[1]-((double)BITS_IN_LONG/K)*dec-2);
+	    modifie=mpadd(x,arrondi);
+	  }
+	  else  modifie=x;
+	  if(expo(modifie)>=((lg(modifie)-2)<<TWOPOTBITS_IN_LONG))
 	    ecrireall(x,'e',dec,chmp,flbl);
 	  else
 	  {
-	    avmacourant=avma;
-	    if (sgn!=1) pariputc('-');
-		/*  on arrondit si il y a lieu */
-	    for (i=0;i<=2;i++) arrondi[i]=x[i];
-	    setlg(arrondi,3);
-	    if (dec>0)
-	    {
-	      arrondi[1]=(long)(arrondi[1]-((double)BITS_IN_LONG/K)*dec-2);
-	      modifie=mpadd(x,arrondi);
-	    }
-	    else  modifie=x;
-	    
 		/* partie entiere */
 	    enti=gcvtoi(modifie,&e);
 	    if(e>0) err(talker,"bug in ecrireall");
@@ -1612,6 +1611,6 @@ outbeauterr(GEN x)
 void
 printversion(void)
 {
-  pariputs("            GP/PARI CALCULATOR Version 1.39\n");
+  pariputs("            GP/PARI CALCULATOR Version 1.39.03\n");
   pariputs(pariversion);
 }
