@@ -1355,9 +1355,10 @@ dbltor(double x)
 #else
   z=cgetr(4);
 #endif
-  ex=(long)floor(log2(x));
-  if(x<0) {z[1]=evalexpo(ex)+evalsigne(-1);x= -x;}
-  else z[1]=evalexpo(ex)+evalsigne(1);
+
+
+  if(x<0) {ex=(long)floor(log2(-x));z[1]=evalexpo(ex)+evalsigne(-1);x= -x;}
+  else {ex=(long)floor(log2(x));z[1]=evalexpo(ex)+evalsigne(1);}
   x=x*exp2(-ex+co-1);y=(ulong)x;z[2]=y;
 #ifdef LONG_IS_32BIT
   x-=y;y=(ulong)(exp2(co)*x);z[3]=y;
