@@ -244,7 +244,7 @@ gasin(GEN x, long prec)
       else
       {
 	tetpil=avma;y=cgetg(3,6);p1=mpach(z);
-	setsigne(p1,-signe(p1));y[2]=(long)p1;
+	y[2]=(long)p1;
 	y[1]=lmppi(lg(x));
 	setexpo((GEN)y[1],0);
 	if (signe(x)<0) gnegz(y,y);
@@ -550,7 +550,7 @@ garg(GEN x, long prec)
     case 1 :
     case 4 :
     case 5 :
-      if(signe(x)>0)
+      if(gsigne(x)>0)
       {
 	y=cgetr(3);y[1]=HIGHEXPOBIT-((prec-2)<<TWOPOTBITS_IN_LONG);
 	y[2]=zero;
@@ -825,17 +825,24 @@ mpash(GEN x)
 GEN
 gash(GEN x, long prec)
 {
-  long    av,tetpil,v;
+  long    av,tetpil,v,sx,sy,sz;
   GEN     y,p1;
 
+  if(gcmp0(x)) return gcopy(x);
   switch(typ(x))
   {
     case 2 : y=mpash(x);break;
     
     case 6 : av=avma;p1=gaddsg(1,gmul(x,x));
       p1=gadd(x,gsqrt(p1,prec));
-      p1=glog(gmul(p1,p1),prec);tetpil=avma;
-      y=gerepile(av,tetpil,gmul2n(p1,-1));
+      tetpil=avma;y=glog(p1,prec);sz=gsigne((GEN)y[1]);
+      sx=gsigne((GEN)p1[1]);sy=gsigne((GEN)p1[2]);
+      if((sx>0)||((!sx)&&(sy*sz<=0))) return gerepile(av,tetpil,y);
+      else
+      {
+	y=gneg(y);p1=cgetg(3,6);p1[1]=zero;p1[2]=(long)mppi(prec);tetpil=avma;
+	return gerepile(av,tetpil,(sy>=0)?gadd(y,p1):gsub(y,p1));
+      }
       break;
     
     case 3 :
@@ -1473,12 +1480,12 @@ cxlngamma(GEN x, long prec)
     subrsz((GEN)p2[1],1,(GEN)p2[1]);gmulz(p8,p2,p8);
   }
   gsubz(p4,glog(p8,l+1),p4);
-  if (flag)
-  {
-    pitemp=mppi(l+1);gmulz(pitemp,x,p1);gdivz(pitemp,gsin(p1,l+1),p1);
-    gsubz(glog(p1,l+1),p4,y);
-  }
+  pitemp=mppi(l+1);
+  if (flag) gsubz(glog(gdiv(pitemp,gsin(gmul(pitemp,x),l+1)),l+1),p4,y);
   else gaffect(p4,y);
+  p1=gsub(pitemp,(GEN)y[2]);setexpo(pitemp,2);
+  gaddz(gmul(gfloor(gdiv(p1,pitemp)),pitemp),(GEN)y[2],(GEN)y[2]);
+  setexpo(pitemp,1);
   avma=av;return y;
 }
     
